/***********************************************/
/* Cleaning a Network in Parallel with Brushes */
/* Bipartite graphs                            */
/* Pawel Pralat (11-Mar-2008)                  */
/***********************************************/
#include <stdio.h>

class graph
{
public:

  int n;
  int flag_seq;
  int flag_par;
  int ** g;
  int * d;
  int e;
  int * conf;
  int * ini_conf;
  int * not_fired;
  int * d_clean;

  graph(char * c);

  ~graph();
  int check();
  int check_par();
  void print_graph();
  void gen_conf(int k, int start);
};

graph::graph(char * c)
{
  int i, j;

  n = *c-63;
  e = 0;
  c++;

  g = new int * [n];
  d = new int [n];
  d_clean = new int [n];
  conf = new int [n];
  ini_conf = new int [n];
  not_fired = new int [n];

  for (i = 0 ; i < n ; i++)
  {
    g[i] = new int [n];
    d[i] = 0;
    ini_conf[i] = 0;
  }

  int bit = 5;
  int number = *c-63;
  c++;

  for (i = 1 ; i < n ; i++)
  {
    for (j = 0 ; j < i ; j++)
      {
	if (bit == -1)
	  {
            bit = 5;
            number = *c-63;
            c++;
	  }

        g[j][i] = g[i][j] = (number >> bit) & 1;
        if (g[j][i])
        {
          e++;
          d[i]++;
          d[j]++;
        }
	bit--;
      }
  }
}

graph::~graph()
{
  for (int i = 0 ; i < n ; i++)
    delete [] g[i];

  delete [] g;
  delete [] d;
  delete [] conf;
  delete [] ini_conf;
  delete [] d_clean;
  delete [] not_fired;
}

void graph::print_graph()
{
  printf("%c", n+63);

  int bit = 0;
  int number = 0;

  int i, j;
  for (i = 1 ; i < n ; i++)
  {
    for (j = 0 ; j < i ; j++)
      {
	bit++;
	number <<= 1;
	if (g[i][j] == 1)
	  number++;

	if (bit == 6)
	  {
	    printf("%c", number+63);
	    bit = number = 0;
	  }
      }
  }

  if (bit != 0)
    {
      number <<= (6-bit);
      printf("%c", number+63);
    }
  printf("\n");
}

int graph::check_par()
{
  int i, j;

  for (i = 0 ; i < n ; i++)
    conf[i] = ini_conf[i];

  int * ready = new int [n];

for (int iter = 0 ; iter < 20 ; iter++)
{
  int nr_not_fired = n;
  int nr_prev_not_fired = n;

  for (i = 0 ; i < n ; i++)
  {
    d_clean[i] = d[i];
    not_fired[i] = 1;
  }

  while (nr_not_fired)
  {
    for (i = 0 ; i < n ; i++)
      if ((not_fired[i]) && (conf[i] >= d_clean[i]))
      {
        ready[nr_prev_not_fired-nr_not_fired] = i;
        nr_not_fired--;
      }

    for (int k = 0 ; k < nr_prev_not_fired-nr_not_fired ; k++)
    {
      i = ready[k];
      for (j = 0 ; j < n ; j++)
        if ((g[i][j]) && (not_fired[j]) && (i!=j))
        {
          conf[j]++;
          d_clean[j]--;
          conf[i]--;
        }
    }

    for (j = 0 ; j < nr_prev_not_fired-nr_not_fired ; j++)
    {
      i = ready[j];
      not_fired[i] = 0;
    }

    if (nr_not_fired == nr_prev_not_fired)
    {
      delete [] ready;
      return 0;
    }
    else
      nr_prev_not_fired = nr_not_fired;
  }
}

  flag_par = 1;
  delete [] ready;
  return 1;
}

int graph::check()
{
  int i, j;
  int nr_not_fired = n;
  int nr_prev_not_fired = n;

  for (i = 0 ; i < n ; i++)
  {
    conf[i] = ini_conf[i];
    d_clean[i] = d[i];
    not_fired[i] = 1;
  }

  while (nr_not_fired)
  {
    for (i = 0 ; i < n ; i++)
      if ((not_fired[i]) && (conf[i] >= d_clean[i]))
        {
          not_fired[i] = 0;
          nr_not_fired--;
          conf[i] -= d_clean[i];
          for (j = 0 ; j < n ; j++)
            if ((g[i][j]) && (not_fired[j]))
              {
                conf[j]++;
                d_clean[j]--;
              }
        }

    if (nr_not_fired == nr_prev_not_fired)
      return 0;
    else
      nr_prev_not_fired = nr_not_fired;
  }

  flag_seq = 1;
  return 1;
}

void graph::gen_conf(int k, int start)
{
  if (start == n-1)
    {
      ini_conf[start] = k;

      if (check())
	{
	  for (int i = 0 ; i < n ; i++)
	    printf("%d", ini_conf[i]);
	  printf(" ");
	}


      if (check())
	if (!check_par())
	  printf("!");
    }
  else
    {
      // for (int i = 0 ; (i <= k) && ((!flag_seq) || (!flag_par)) ; i++)
      for (int i = 0 ; i <= k ; i++)
	{
	  ini_conf[start]=i;
	  gen_conf(k-i, start+1);
	}
    }
}

int main(int argc, char * argv[])
{
  char graph_text [100];

  FILE * in = fopen(argv[1], "rt");
  while (!feof(in))
  {
    fscanf(in, "%s\n", graph_text);
    graph g(graph_text);

    g.flag_seq = g.flag_par = 0;
    int k = 0;
    while (!g.flag_seq)
    {
      k++;
      g.gen_conf(k, 0);
    }

    printf("%d ", k);
    if (!g.flag_par)
      printf(" **NO** ");
    g.print_graph();
  }

  return 0;
}
