/***********************************************/
/* Cleaning a Network in Parallel with Brushes */
/* Cliques                                     */
/* Pawel Pralat (11-Mar-2008)                  */
/***********************************************/
#include <stdio.h>
#include <stdlib.h>

int k;                           // the size of the clique
const int max_iterations = 20;

// change this if you want to find the number for cliques of size > 40
int initial_configuration[40];
int conf[40];
int dirty[40];

long long number_of_configurations;

int check_configuration(int action)
{
  int i, nr_of_dirty, nr_to_clean;
  long j;
  for (i = 0 ; i < k ; i++)
    conf[i] = initial_configuration[i];

  for (j = 0 ; j < max_iterations ; j++)
    {
      if (action)
	{
	  printf("%d ) ", j);
	  for (i = 0 ; i < k ; i++)
	    printf("%d ", conf[i]);
	  printf("\n");
	}

      nr_of_dirty = k;
      for (i = 0 ; i < k ; i++)
	dirty[i] = 1;

      while (nr_of_dirty)
	{
	  nr_to_clean = 0;
	  for (i = 0 ; i < k ; i++)
	    if ((dirty[i]) && (conf[i]>=nr_of_dirty-1))
	      nr_to_clean++;

	  if (nr_to_clean == 0)
	    return 0;

	  for (i = 0 ; i < k ; i++)
	    if (dirty[i])
	      {
		if (conf[i]>=nr_of_dirty-1)
		  {
		    dirty[i]=0;
		    conf[i] -= nr_of_dirty-nr_to_clean;
		  }
		else
		  conf[i] += nr_to_clean;
	      }

	  nr_of_dirty -= nr_to_clean;
	}
    }
  return 1;
}

void generate_configurations(int coordinate, int nr_brushes_left)
{
  int i;
  if (nr_brushes_left == 0)
    {                                          // we have a new initial configuration!
      if (k-1 - 2*coordinate > 0)
	return;                                // this configuration will not work

      number_of_configurations++;

      for (i = coordinate ; i < k ; i++)
	initial_configuration[i] = 0;

      if (check_configuration(0))
	{
	  check_configuration(1);
	}

      return;
    }

  if (coordinate == k)
    return;

  i = nr_brushes_left;

  if (coordinate && (initial_configuration[coordinate-1] < i))
    i = initial_configuration[coordinate-1];

  int lower_bound = k-1 - 2*coordinate;
  if (lower_bound < 0)
    lower_bound = 0;

  for ( ; i >= lower_bound ; i--)
    {
      initial_configuration[coordinate] = i;
      generate_configurations(coordinate+1, nr_brushes_left - i);
    }
}

int main(int argc, char * argv[])
{
  k = atoi(argv[1]);
  int b = atoi(argv[2]);

  number_of_configurations = 0;
  initial_configuration[0]=k-1;
  generate_configurations(1, b-(k-1));
  printf("number of possible initial configurations : %lld \n", number_of_configurations);
  return 0;
}
